﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Clients.Hive {
  public class EngineHiveJob : HiveJob<EngineJob> {
    private IScope parentScopeClone;

    #region Constructors and cloning
    public EngineHiveJob() { }
    public EngineHiveJob(EngineJob engineJob, IScope parentScopeClone)
      : base(engineJob) {
      this.parentScopeClone = parentScopeClone;
    }

    protected EngineHiveJob(EngineHiveJob original, Cloner cloner)
      : base(original, cloner) {
      this.parentScopeClone = cloner.Clone(original.parentScopeClone);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EngineHiveJob(this, cloner);
    }
    #endregion

    public override JobData GetAsJobData(bool withoutChildOptimizers, out List<IPluginDescription> plugins) {
      plugins = new List<IPluginDescription>();
      if (this.itemJob == null)
        return null;

      JobData jobData = new JobData();
      IEnumerable<Type> usedTypes;

      // clone operation and remove unnecessary scopes; don't do this earlier to avoid memory problems
      lock (locker) {
        ((IAtomicOperation)ItemJob.InitialOperation).Scope.Parent = parentScopeClone;
        ItemJob.InitialOperation = (IOperation)ItemJob.InitialOperation.Clone();
        ((IAtomicOperation)ItemJob.InitialOperation).Scope.ClearParentScopes();
        jobData.Data = PersistenceUtil.Serialize(ItemJob, out usedTypes);
      }

      PluginUtil.CollectDeclaringPlugins(plugins, usedTypes);

      return jobData;
    }
  }
}
