﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Clients.Hive.ExperimentManager;
using HeuristicLab.Clients.Hive.Jobs;
using HeuristicLab.Hive;

namespace HeuristicLab.Clients.Hive {
  public class HiveJobDownloader {
    private IEnumerable<Guid> jobIds;
    private ConcurrentJobDownloader<ItemJob> jobDownloader;
    private IDictionary<Guid, HiveJob> results;

    public bool IsFinished {
      get {
        return results.Count == jobIds.Count();
      }
    }

    public int FinishedCount {
      get {
        return results.Count;
      }
    }

    public IDictionary<Guid, HiveJob> Results {
      get {
        return results;
      }
    }

    public HiveJobDownloader(IEnumerable<Guid> jobIds) {
      this.jobIds = jobIds;
      this.jobDownloader = new ConcurrentJobDownloader<ItemJob>(2, 2);
      this.results = new Dictionary<Guid, HiveJob>();
    }

    public void StartAsync() {
      foreach (Guid jobId in jobIds) {
        jobDownloader.DownloadJob(jobId,
          (id, itemJob, exception) => {
            if (exception != null) {
              throw new ConcurrentJobDownloaderException("Downloading job failed", exception);
            }
            Job job = ServiceLocator.Instance.CallHiveService(s => s.GetJob(id));
            if (job != null && itemJob != null) {
              HiveJob hiveJob;
              if (itemJob is OptimizerJob) {
                hiveJob = new OptimizerHiveJob((OptimizerJob)itemJob);
              } else {
                hiveJob = new HiveJob(itemJob, true);
              }
              hiveJob.Job = job;
              this.results.Add(id, hiveJob);
            }
          });
      }      
    }
  }
}
